// Copyright (C) 2000, Edward W. Felten.  All rights reserved.

import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;


public class Account implements Comparable, Serializable {
  private String name;
  private Money  balance;
  private int    accountNum;

  public Account(String name, int pin, Money balance) {
    this.name = name;
    this.balance = balance;
    AccountDB.add(this);       // calls setNum to set accountNum
    try{
      AtmCard.issue(this, pin);
    }catch(IOException x){
      System.err.println("Couldn't create ATM card for new account --- "
			 + x);
    }
  }

  public synchronized void setNum(int num) {
    accountNum = num;
  }

  public int getNum() {
    return accountNum;
  }

  public String getName() {
    return name;
  }

  public Money getBalance() {
    return balance;
  }

  public synchronized void deposit(Money m) {
    balance.add(m);
    fireBalanceChanged();
  }

  public synchronized void withdraw(Money m) 
    throws InsufficientFundsException {
    balance.subtract(m);
    fireBalanceChanged();
  }

  public int compareTo(Object o) {
    Account a = (Account)o;
    return accountNum - a.accountNum;
  }

  public String toString() {
    return "[acct: " + accountNum + " \"" + name + "\" " + balance + "]"; 
  }
    
  //   deal with balance-listeners
  private transient List balListeners;

  public synchronized void addBalanceChangeListener(ChangeListener cl) {
    if(balListeners == null)    balListeners = new ArrayList();
    balListeners.add(cl);
  }

  public synchronized void removeBalanceChangeListener(ChangeListener cl) {
    if(balListeners == null)    balListeners = new ArrayList();
    balListeners.remove(cl);
  }

  public synchronized void fireBalanceChanged() {
    if(balListeners == null)    balListeners = new ArrayList();
    Iterator iter = balListeners.listIterator();
    while(iter.hasNext()){
      ChangeListener cl = (ChangeListener)(iter.next());
      cl.stateChanged(new ChangeEvent(this));
    }
    AccountDB.fireBalanceChanged(this);
  }
}
