// Copyright (C) 2000, Edward W. Felten.  All rights reserved.

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InvalidClassException;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;


public class AtmCard implements Serializable {
  private int acctNum;
  private byte[] auth;

  private AtmCard(int acctNum, int pin) {
    this.acctNum = acctNum;
    this.auth = AtmCardAuth.computeCheckBytes(acctNum, pin);
  }

  public int getNum() {
    return acctNum;
  }

  public byte[] getAuth() {
    return auth;
  }

  public boolean authenticate(int pin) {
    return AtmCardAuth.checkPin(acctNum, pin, auth);
  }

  public boolean authenticate(String pin) {
    try{
      return authenticate(Integer.parseInt(pin));
    }catch(NumberFormatException x){
      return false;
    }
  }

  private static String cardFileName(int num) {
    return "card_" + Integer.toString(num) + ".atm";
  }

  public static AtmCard issue(Account acct, int pin) throws IOException {
    int num = acct.getNum();
    AtmCard card = new AtmCard(num, pin);
    ObjectOutputStream ois = 
      new ObjectOutputStream(new FileOutputStream(cardFileName(num)));
    ois.writeObject(card);
    ois.close();
    return card;
  }

  public static AtmCard fromFile(int num) {
    try {
      ObjectInputStream ois = 
	new ObjectInputStream(new FileInputStream(cardFileName(num)));
      return (AtmCard)(ois.readObject());
    }catch(InvalidClassException x){
      System.out.println("Error: you need to reinitialize the bank!");
      return null;
    }catch(Exception x){
      System.out.println("AtmCard::fromFile: " + x);
      return null;
    }
  }

  public static AtmCard fromFile(Account acct) {
    return fromFile(acct.getNum());
  }
}
