// Copyright (C) 2000, Edward W. Felten.  All rights reserved.


public class Connection {
  private DropBox outgoing;
  private DropBox incoming;

  public Connection() {
  }

  public Connection(Connection partner) {
    outgoing = partner.incoming = new DropBox();
    incoming = partner.outgoing = new DropBox();
  }

  public void send(byte[] msg) throws ConnectionClosedException {
    outgoing.send(msg);
  }

  public byte[] receive() throws ConnectionClosedException {
    return incoming.receive();
  }

  public void close() {
    outgoing.close();
    incoming.close();
  }

  class DropBox {
    private boolean closed = false;
    private byte[] contents = null;

    synchronized void send(byte[] msg) throws ConnectionClosedException {
      if(msg == null)    throw new NullPointerException();
      while( (!closed) && (contents != null)){
	try{
	  wait();
	}catch(InterruptedException x){
	}
      }
      if(closed)    throw new ConnectionClosedException();
      contents = msg;
      notifyAll();
    }

    synchronized byte[] receive() throws ConnectionClosedException {
      // Note: we check for a closed connection inside the loop.
      // This ensures that if the dropbox is closed while there's still 
      //   something in it, receiver will get that something before
      //   he's told the connection has closed.
      while(contents == null){
	if(closed)    throw new ConnectionClosedException();
	try{
	  wait();
	}catch(InterruptedException x){
	}
      }
      byte[] ret = contents;
      contents = null;
      notifyAll();
      return ret;
    }

    synchronized void close() {
      closed = true;
    }
  }
}
